#! /usr/bin/python
# -*- coding: utf8 -*-

# Time filtering in time domain for improved handling of gp2x's stick
# by Radek Kowalski 2006
# This is 3rd version and is quite different than previous.
# (actually it's 4th but I decided to public the "real 3rd" as it was
# to complicated to be able to comprehend even by me) ;)
# Public Domain and use at your own risk.
#
# Below (if not stated differently) is supporting code of this
# program to just demonstrate the effects of algorithm.


# GP2X joystick button mappings
BUTTON_UP            = (0)
BUTTON_DOWN          = (4)
BUTTON_LEFT          = (2)
BUTTON_RIGHT         = (6)
BUTTON_UPLEFT        = (1)
BUTTON_UPRIGHT       = (7)
BUTTON_DOWNLEFT      = (3)
BUTTON_DOWNRIGHT     = (5)
BUTTON_CLICK         = (18)
BUTTON_A             = (12)
BUTTON_B             = (13)
BUTTON_X             = (14)
BUTTON_Y             = (15)
BUTTON_L             = (10)
BUTTON_R             = (11)
BUTTON_START         = (8) 
BUTTON_SELECT        = (9)
BUTTON_VOLUP         = (16)
BUTTON_VOLDOWN       = (17)

from pygame import *

display.init()
scr = display.set_mode((320, 240))

background = image.load("timegate3.png")
scr.blit(background, (0, 0))

# initialisation
joystick.init()
num_joysticks = joystick.get_count()
if num_joysticks > 0:
    stick = joystick.Joystick(0)
    stick.init() # now we will receive events for the joystick

def SetStick(StickButtons, ev):
    if ev.button == BUTTON_UP:   Switch = 0
    elif ev.button == BUTTON_DOWN: Switch = 4
    elif ev.button == BUTTON_LEFT:  Switch = 6
    elif ev.button == BUTTON_RIGHT: Switch = 2
    elif ev.button == BUTTON_UPLEFT: Switch = 7
    elif ev.button == BUTTON_UPRIGHT: Switch = 1
    elif ev.button == BUTTON_DOWNLEFT: Switch = 5
    elif ev.button == BUTTON_DOWNRIGHT:  Switch = 3
    elif ev.button == BUTTON_CLICK: Switch = 8
    else:
        return 0
    if ev.type == JOYBUTTONDOWN:
        StickButtons[Switch] = 1
    elif ev.type == JOYBUTTONUP:
        StickButtons[Switch] = 0
    else:
        StickButtons = [0, 1, 0, 1, 0, 1, 0, 1, 0]
        return 0
    
def SetStickWithKeyboard(StickButtons, ev):
    """This procedure is very usefull to debugging code on a PC."""
    if ev.key == 119:   Switch = 0
    elif ev.key == 120: Switch = 4
    elif ev.key == 97:  Switch = 6
    elif ev.key == 100: Switch = 2
    elif ev.key == 113: Switch = 7
    elif ev.key == 101: Switch = 1
    elif ev.key == 122: Switch = 5
    elif ev.key == 99:  Switch = 3
    elif ev.key == 115: Switch = 8
    else:
        return 0
    if ev.type == KEYDOWN:
        StickButtons[Switch] = 1
    elif ev.type == KEYUP:
        StickButtons[Switch] = 0
    else:
        StickButtons = [0, 1, 0, 1, 0, 1, 0, 1, 0]
        return 0

def StickPreFilter(Stick, Out):
    for x in [0, 1, 2, 3, 4, 5, 6, 7, 8]:
        Out[x] = Stick[x]
    if Stick[0] and Stick[1]:
        Out[1] = 0
    elif Stick[1] and Stick[2]:
        Out[1] = 0
    elif Stick[2] and Stick[3]:
        Out[3] = 0
    elif Stick[3] and Stick[4]:
        Out[3] = 0
    elif Stick[4] and Stick[5]:
        Out[5] = 0
    elif Stick[5] and Stick[6]:
        Out[5] = 0
    elif Stick[6] and Stick[7]:
        Out[7] = 0
    elif Stick[7] and Stick[0]:
        Out[7] = 0

def DrawStickPositions(scr, StickButtons, (posX, posY) = (10, 10),
                                rectangleSize = 30, rectangleSpacing = 50):
    StickM = ((7, 0, 1), (6, 8, 2), (5, 4, 3))
    for y in [0, 1, 2]:
        for x in [0, 1, 2]:
            if StickButtons[StickM[y][x]] > 0:
                color = (255, 0, 0)
            else:
                color = (255, 255, 255)
            draw.rect(scr, color, (x * rectangleSpacing + posX,
                                             y * rectangleSpacing + posY,
                                             rectangleSize, rectangleSize))

def DrawStickModes(scr, FData, (posX, posY), RectSize, RectSpacing):
    StickM = ((7, 0, 1), (6, -1, 2), (5, 4, 3))
    for y in [0, 1, 2]:
        for x in [0, 1, 2]:
            i = StickM[y][x]
            if i == -1:
                continue
            color = ((255, 0, 0), (0, 255, 0), (0, 0, 255))[FData[i][6]]
            draw.rect(scr, color, (x * RectSpacing + posX,
                                    y * RectSpacing + posY,
                                        RectSize, RectSize))

def DrawVerticalBar(scr, Val, MaxVal, (posX, posY), recSize, recSpacing):
    for y in range(MaxVal):
        if (MaxVal - y - 1) < Val:
            draw.rect(scr, (255, 0, 0), (posX, posY + recSpacing * y, recSize, recSize), 0)
        else:
            draw.rect(scr, (255, 255, 255), (posX, posY + recSpacing * y, recSize, recSize), 0)

# Below are procedures what are the core of that application. The rest is just
# supporting code to show up the idea.

def SwitchGate(GateVector, Input):
    # This procedure is making most of the work...
    # It works for one button and is performing dynamic gating with
    # settable parameters with three parameters. For proper filtering
    # of gp2x's stick it must be called 8 times for 8 different vectors
    # per one polling cycle.
    # GateVector is like:
    #       [PreAttack, PostAttack, PostRelase, PreAtkCounter, PostAtkCounter,
    #                                         PostRleCounter, Mode]
    # Input is 0 or 1 depending if event was buttondown or buttonup.
    # The "mode" when it's 0 then gate is "Pre-mode", 1 then "Carry",
    # 2 then "Loop".
    GV = GateVector     # have enough of longer names
    GV[3] -= (GV[3] > 0) and 1     # simple yet effective way of decreasing
    GV[4] -= (GV[4] > 0) and 1     # counters without fear of going to minus
    GV[5] -= (GV[5] > 0) and 1     # values
    # There there states of this gate - Pre, Carry and Loop.
    # Pre is when the on event is just showing and wasn't long enough
    # to get past the PreAttack. It's then passed to Output without any
    # filtering.
    # Carry is when the on event was long enough to get past PreAttack.
    # It's really into halfstep of the next mode (the "Loop").
    # Loop is mode when events always be filtered except the first time
    # (the "Carry") and gate will return to the "Pre" mode if there will not be
    # on event in time of PostRelase.

    Output = 0      # unnecessary however to be safe...

    if GV[0] <= 1:  # to allow to jump from Pre to Loop mode directly
        GV[6] = 2   # could be done in more elegant way...
    # The "Pre-mode" code.
    if GV[6] == 0:
        if Input == 0:
            GV[3] = GV[0]
            GV[6] = 0
        elif GV[3] == 0:
            GV[6] = 1
        Output = Input
    # The "Carry-mode"
    elif GV[6] == 1:
        if Input == 0:
            GV[5] = GV[2]
            GV[4] = GV[1]   # this line might be unnecessary here
            GV[6] = 2
            Output = 0
        else:
            Output = 1
    # The "Loop-mode"
    elif GV[6] == 2:
        if Input == 1:
            GV[5] = GV[2]
        if GV[5] == 0:
            GV[6] = 0
        if Input == 0:
            GV[4] = GV[1]
        if GV[4] == 0 and Input == 1: # not optimally done...
            Output = 1
        else:
            Output = 0
    # All code above was written having the Arm cpu in a mind.
    # This is why there are so many conditionals (the "ifs") as Arm
    # likes them very much thanks to its conditional execution.
    # Converting it to the C or even Asm should be quite straightforward.
    # Yet it has some unnecessary instructions, could be shorter...
    # but no Python craziness here! ;)
    return Output

# Back to supporting code...

def ComputeAllVectors(Vectors, Inputs, Outputs):
    for x in (0, 1, 2, 3, 4, 5, 6, 7):
        Outputs[x] = SwitchGate(Vectors[x], Inputs[x])
    Outputs[8] = Inputs[8] # the stick push down event isn't filtered at all


FilterData = [[0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0]]


StickButtons = [0, 0, 0, 0, 0, 0, 0, 0, 0]
StOutput = [0, 0, 0, 0, 0, 0, 0, 0, 0]
FilteredOutput = [0, 0, 0, 0, 0, 0, 0, 0, 0]

# Parameters I will allow to tweak in a demo.
ParVector = [0, 0, 0, 0, 0, 0]

def SetParameters():
    global FilterData
    for x in (0, 1, 2, 3, 4, 5, 6, 7):
        if x % 2 > 0:   # for diagonals
            FilterData[x][0] = ParVector[3]
            FilterData[x][1] = ParVector[4]
            FilterData[x][2] = ParVector[5]
        else:           # for ordinals
            FilterData[x][0] = ParVector[0]
            FilterData[x][1] = ParVector[1]
            FilterData[x][2] = ParVector[2]


# Six of them so twice than before, soon I will have not
# a single button left on the gp2x...
# (there are 8 * 3 = 24 parameters really but I decided to
#  give only option to set the ordinals and diagonals, no
#  tweaking for individual switches but it could be done)


SetParameters()

SelectedOption = 0
NextOption = 0

while True:
    try:
        e = event.poll()
        if e.type != NOEVENT:
            if e.type == JOYBUTTONDOWN:
                if e.button == BUTTON_SELECT or e.button == BUTTON_START:
                    break
                elif e.button == BUTTON_A:
                    NextOption -= 1
                elif e.button == BUTTON_B:
                    NextOption += 1
                elif e.button == BUTTON_Y:
                    value = 1
                elif e.button == BUTTON_X:
                    value = -1
                else:
                    value = 0

                if NextOption > 5:
                    NextOption = 0
                if NextOption < 0:
                    NextOption = 5

                if value != 0:
                    ParVector[SelectedOption] += value
                    if ParVector[SelectedOption] < 1:
                        ParVector[SelectedOption] = 1
                    elif ParVector[SelectedOption] > 10:
                        ParVector[SelectedOption] = 10
                    SetParameters()

            SetStick(StickButtons, e)

        StickPreFilter(StickButtons, StOutput)

        ComputeAllVectors(FilterData, StOutput, FilteredOutput)

        DrawStickPositions(scr, StickButtons, (8,8), 20, 25)
        DrawStickPositions(scr, StOutput, (127, 8), 20, 25)

        DrawStickPositions(scr, FilteredOutput, (243, 8), 20, 25)
        DrawStickModes(scr, FilterData, (261, 106), 14, 19)

        DrawVerticalBar(scr, ParVector[0] - 1, 9, (31 + 40 * 0, 106), 8, 12)
        DrawVerticalBar(scr, ParVector[1] - 1, 9, (31 + 40 * 1, 106), 8, 12)
        DrawVerticalBar(scr, ParVector[2] - 1, 9, (31 + 40 * 2, 106), 8, 12)
        DrawVerticalBar(scr, ParVector[3] - 1, 9, (31 + 40 * 3, 106), 8, 12)
        DrawVerticalBar(scr, ParVector[4] - 1, 9, (31 + 40 * 4, 106), 8, 12)
        DrawVerticalBar(scr, ParVector[5] - 1, 9, (31 + 40 * 5, 106), 8, 12)

        draw.rect(scr, (255, 255, 255), (3 + 40 * SelectedOption, 104, 38, 108), 1)
        draw.rect(scr, (255, 0, 255), (3 + 40 * NextOption, 104, 38, 108), 1)
        SelectedOption = NextOption

        display.flip()
        #time.wait(100)  # not needed on gp2x but usefull when running on a PC :)
    except:
        pass
